
public class Test21 {

	/**
	 * Construct a tiny binary tree (depth 1, n=4). <br>
	 *     A         <br>
	 *    /|\        <br>
	 *   A B C       <br>
	 */
	public static Tree createTinyTree() {
		Tree root = new Tree("A");
		(root.children).add(new Tree("B")); // depth 1, left child
		(root.children).add(new Tree("C")); // depth 1, right child
		(root.children).add(new Tree("D")); // depth 1, right child

		return root;
	}

	/**
	 * Construct a tiny binary tree (depth 3). <br>
	 *     A         <br>
	 *    / \        <br>
	 *   B   C       <br>
	 *  / \  | \     <br>
	 * D   E F  G    <br>
	 *    /|    |\   <br>
	 *   H I    J K  <br>
	 */
	public static Tree createT1() {
		Tree root = new Tree("A");
		(root.children).add(new Tree("B")); // depth 1, left child
		(root.children).add(new Tree("C")); // depth 1, right child

		(root.children.get(0).children).add(new Tree("D")); // depth 2
		(root.children.get(0).children).add(new Tree("E")); // depth 2
		(root.children.get(1).children).add(new Tree("F")); // depth 2
		(root.children.get(1).children).add(new Tree("G")); // depth 2

		(root.children.get(0).children.get(1).children).add(new Tree("H")); // depth 3
		(root.children.get(0).children.get(1).children).add(new Tree("I")); // depth 3
		(root.children.get(1).children.get(1).children).add(new Tree("J")); // depth 3
		(root.children.get(1).children.get(1).children).add(new Tree("K")); // depth 3

		return root;
	}

	/**
	 * Construct a tiny tree (height 3). <br>
	 *        A            <br>
	 *    / | \  \         <br>
	 *   B  C  D    E      <br>
	 *  / \    |  / | \    <br>
	 * F   G   H  I J K    <br>
	 *     |               <br>
	 *     L
	 */
	public static Tree createT2() {
		Tree root = new Tree("A"); // root
		(root.children).add(new Tree("B")); // depth 1
		(root.children).add(new Tree("C")); // depth 1
		(root.children).add(new Tree("D")); // depth 1
		(root.children).add(new Tree("E")); // depth 1
		(root.children.get(0).children).add(new Tree("F")); // depth 2
		(root.children.get(0).children).add(new Tree("G")); // depth 2
		(root.children.get(2).children).add(new Tree("H")); // depth 2
		(root.children.get(3).children).add(new Tree("I")); // depth 2
		(root.children.get(3).children).add(new Tree("J")); // depth 2
		(root.children.get(3).children).add(new Tree("K")); // depth 2

		(root.children.get(0).children.get(1).children).add(new Tree("L")); // depth 3

		return root;
	}

	public static void main(String[] args) {
		if (!Test21.class.desiredAssertionStatus()) {
			System.err.println("Vous devez activer l'option -ea de la JVM");
			System.err.println("To run the tests please activate the -ea option of the JVM");
			System.exit(1);
		}
		
		Tree t, res;
		System.out.println("(Q2.1) Validité du calcul du LCA: Test de GR.lca()");

		System.out.print("\t Teste arbre de hauteur 1 (les 3 fils de la racine sont B, C, D) ...");
		t=createTinyTree();
		res=Tree.lca(t, "A", "B");
		assert (res.element.equals("A")): "\nLe LCA pour la paire (A, B) devrait etre A, au lieu de "+res;
		res=Tree.lca(t, "B", "D");
		assert (res.element.equals("A")): "\nLe LCA pour la paire (B, D) devrait etre A, au lieu de "+res;
		res=Tree.lca(t, "D", "C");
		assert (res.element.equals("A")): "\nLe LCA pour la paire (D, C) devrait etre A, au lieu de "+res;
		res=Tree.lca(t, "B", "C");
		assert (res.element.equals("A")): "\nLe LCA pour la paire (B, C) devrait etre A, au lieu de "+res;
		
		System.out.println("\t\t [OK].");

		System.out.print("\t Teste arbre binaire de hauteur 3 (11 noeuds) ...");
		t=createT1();
		res=Tree.lca(t, "D", "I");
		assert (res.element.equals("B")): "\nLe LCA pour la paire (D, I) devrait etre B, au lieu de "+res;
		res=Tree.lca(t, "H", "D");
		assert (res.element.equals("B")): "\nLe LCA pour la paire (H, D) devrait etre B, au lieu de "+res;
		res=Tree.lca(t, "I", "K");
		assert (res.element.equals("A")): "\nLe LCA pour la paire (I, K) devrait etre A, au lieu de "+res;
		res=Tree.lca(t, "F", "J");
		assert (res.element.equals("C")): "\nLe LCA pour la paire (F, G) devrait etre G, au lieu de "+res;
		
		System.out.println("\t\t\t\t [OK].");

		System.out.print("\t Teste l'arbre de l'exemple fourni dans le sujet (12 noeuds) ...");
		t=createT2();
		//GT.print(t); // print the input tree for debugging

		res=Tree.lca(t, "A", "B");
		assert (res.element.equals("A")): "\nLe LCA pour la paire (A, B) devrait etre A, au lieu de "+res;
		res=Tree.lca(t, "E", "B");
		assert (res.element.equals("A")): "\nLe LCA pour la paire (E, B) devrait etre A, au lieu de "+res;
		res=Tree.lca(t, "H", "E");
		assert (res.element.equals("A")): "\nLe LCA pour la paire (H, E) devrait etre A, au lieu de "+res;
		res=Tree.lca(t, "C", "E");
		assert (res.element.equals("A")): "\nLe LCA pour la paire (C, E) devrait etre A, au lieu de "+res;
		res=Tree.lca(t, "F", "D");
		assert (res.element.equals("A")): "\nLe LCA pour la paire (F, D) devrait etre A, au lieu de "+res;
		res=Tree.lca(t, "K", "C");
		assert (res.element.equals("A")): "\nLe LCA pour la paire (K, C) devrait etre A, au lieu de "+res;
		res=Tree.lca(t, "J", "L");
		assert (res.element.equals("A")): "\nLe LCA pour la paire (J, L) devrait etre A, au lieu de "+res;

		res=Tree.lca(t, "F", "L");
		assert (res.element.equals("B")): "\nLe LCA pour la paire (J, L) devrait etre B, au lieu de "+res;
		res=Tree.lca(t, "F", "B");
		assert (res.element.equals("B")): "\nLe LCA pour la paire (F, B) devrait etre B, au lieu de "+res;
		res=Tree.lca(t, "G", "F");
		assert (res.element.equals("B")): "\nLe LCA pour la paire (G, F) devrait etre B, au lieu de "+res;

		res=Tree.lca(t, "I", "K");
		assert (res.element.equals("E")): "\nLe LCA pour la paire (I, K) devrait etre E, au lieu de "+res;

		System.out.println("\t\t [OK].");
	}

}
